# Learning Object-aware Anchor-free Networks for Real-time Object Tracking (OceanPlus)

- If you have any issue with running our tracker, please feel free to contact zhangzhipeng2017@ia.ac.cn .

## Important Files 
- OceanPlus/vot_OceanPlus.py <br/>
  The interface between our tracker with VOT2020 python toolkit.
- OceanPlus/install.sh <br/>
  The guidance of conda environment and python packages installation. 
- OceanPlus/lib <br/>
  Main Code of our tracker.
- OceanPlus/experiment <br/>
  Hyperparameters of our tracker.
- OceanPlus/snapshot  <br/>
  Pretrained models (empty).


## Running Environment
python3.7 <br/>
pytorch1.1.0 <br/>
cudatoolkit10.0.130 <br/>
Nvidia RTX2080Ti (12GB) <br/>
Intel(R) Xeon(R) Gold 5118 CPU @ 2.30GHz <br/>

## Results (EAO)
| baseline | realtime | 
| ------ | ------ | 
|0.500	| 0.484 | 


## Run Tracker
If you couldn't run our tracker successfully, please first check the `Note` in the last section of this guidance. 

### Step1. Set up the environment
Before setting up the environment, please check the CUDA version in your machine. If you are using CUDA9, please modify the line23 in `OceanPlus/install.sh` to `conda install -y pytorch==1.1.0 cudatoolkit=9.0 -c pytorch` first. We use CUDA10 in our submission.

In the code (OceanPlus),
```
bash install.sh $conda_path OceanPlus
```
`$conda_path` indicates your anaconda installation path, e.g. `/data/zpzhang/anaconda3`.

### Step2. Set up VOT2020 toolkit
step1 will automatically download vot toolkit.
```
conda activate OceanPlus
mkdir vot-workspace
cd vot-workspace
vot workspace vot2020 --workspace <workspace-path>
cp $tracker_path/trackers.ini .
```
`<workspace-path>` indicates the absolute path of `vot-workspace`, e.g. `/data/zpzhang/vot-workspace`. `tracker_path` indicates the absolute path of our tracker, e.g. `/data/zpzahng/OceanPlus`.

### Step3. Download the pre-trained model
Please download the pre-trained model [here](https://drive.google.com/open?id=1nLO9P6VvbYrj9HYxLKzk-AO51A3EaXEa). And then put it into `$tracker_path/snapshot/`

### Step4. Modify path in codes
- Modify `$tracker_path` of line7 in `<workspace-path>/trackers.ini`, e.g. `/data/zpzahng/OceanPlus`.
- Modify `$tracker_path` of line88 in `$tracker_path/vot_OceanPlus.py`, e.g. `/data/zpzahng/OceanPlus`.

### Step5. Run testing
In `vot-workspace`,
```
conda activate OceanPlus
CUDA_VISIBLE_DEVICES=$gpu_id vot evaluate --workspace <workspace-path> OceanPlus
```
`$gpu_id` indicates the ID of GPU that you plan to run the tracker.


### Step6. Get results
In `vot-workspace`,
```
vot analysis --workspace <workspace-path> OceanPlus --output json 
```

### Note
- Before running our tracker, you should consider temporally stop other programs that take much CPU or GPU resources. Otherwise, it may bring negative effect on realtime running. If you only run the baseline experiment, please ignore this. 
- We run our tracker with RTX 2080Ti. The GPU with worse capacity will degradate reatime results. 
- If you met the error `PermissionError: [Errno 13] Permission denied: '/tmp/torch_extensions/_prroi_pooling/lock'`, please remove `/tmp/torch_extensions/ _prroi_pooling` and rerun the tracker. If other user in your machine have compiled prroi pooling before, this may happens. Besides, if you have compiled pproi_pooling before, please remove `/tmp/torch_extensions/`. Otherwise, you may fail to compile in the new conda environment.
- If your tracker stopped with no clear information in the log, and the log stops at `Using /tmp/torch_extensions as PyTorch extensions root...`, please remove `/tmp/torch_extensions`.

- If you met the error `ImportError: libcudart.so.10.0: cannot open shared object file: No such file or directory`, please run `sudo cp /usr/local/cuda-10.0/lib64/libcudart.so.10.0 /usr/local/lib/libcudart.so.10.0 && sudo ldconfig`.
